package java.awt.geom;

import java.io.Serializable;
import java.awt.Rectangle;
import java.awt.Shape;

public abstract class Line2D implements Shape, Cloneable
{
    protected Line2D() {
    }
    
    public abstract double getX1();
    
    public abstract double getY1();
    
    public abstract Point2D getP1();
    
    public abstract double getX2();
    
    public abstract double getY2();
    
    public abstract Point2D getP2();
    
    public abstract void setLine(double p0, double p1, double p2, double p3);
    
    public void setLine(Point2D point2D, Point2D point2D2) {
        this.setLine(point2D.getX(), point2D.getY(), point2D2.getX(), point2D2.getY());
    }
    
    public void setLine(Line2D line2D) {
        this.setLine(line2D.getX1(), line2D.getY1(), line2D.getX2(), line2D.getY2());
    }
    
    public static int relativeCCW(double n, double n2, double n3, double n4, double n5, double n6) {
        n3 -= n;
        n4 -= n2;
        n5 -= n;
        n6 -= n2;
        double n7 = n5 * n4 - n6 * n3;
        if (n7 == 0.0) {
            n7 = n5 * n3 + n6 * n4;
            if (n7 > 0.0) {
                n5 -= n3;
                n6 -= n4;
                n7 = n5 * n3 + n6 * n4;
                if (n7 < 0.0) {
                    n7 = 0.0;
                }
            }
        }
        return (n7 < 0.0) ? -1 : (n7 > 0.0);
    }
    
    public int relativeCCW(double n, double n2) {
        return relativeCCW(this.getX1(), this.getY1(), this.getX2(), this.getY2(), n, n2);
    }
    
    public int relativeCCW(Point2D point2D) {
        return relativeCCW(this.getX1(), this.getY1(), this.getX2(), this.getY2(), point2D.getX(), point2D.getY());
    }
    
    public static boolean linesIntersect(double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8) {
        return relativeCCW(n, n2, n3, n4, n5, n6) * relativeCCW(n, n2, n3, n4, n7, n8) <= 0 && relativeCCW(n5, n6, n7, n8, n, n2) * relativeCCW(n5, n6, n7, n8, n3, n4) <= 0;
    }
    
    public boolean intersectsLine(double n, double n2, double n3, double n4) {
        return linesIntersect(n, n2, n3, n4, this.getX1(), this.getY1(), this.getX2(), this.getY2());
    }
    
    public boolean intersectsLine(Line2D line2D) {
        return linesIntersect(line2D.getX1(), line2D.getY1(), line2D.getX2(), line2D.getY2(), this.getX1(), this.getY1(), this.getX2(), this.getY2());
    }
    
    public static double ptSegDistSq(double n, double n2, double n3, double n4, double n5, double n6) {
        n3 -= n;
        n4 -= n2;
        n5 -= n;
        n6 -= n2;
        double n8;
        if (n5 * n3 + n6 * n4 <= 0.0) {
            n8 = 0.0;
        }
        else {
            n5 = n3 - n5;
            n6 = n4 - n6;
            double n7 = n5 * n3 + n6 * n4;
            if (n7 <= 0.0) {
                n8 = 0.0;
            }
            else {
                n8 = n7 * n7 / (n3 * n3 + n4 * n4);
            }
        }
        double n9 = n5 * n5 + n6 * n6 - n8;
        if (n9 < 0.0) {
            n9 = 0.0;
        }
        return n9;
    }
    
    public static double ptSegDist(double n, double n2, double n3, double n4, double n5, double n6) {
        return Math.sqrt(ptSegDistSq(n, n2, n3, n4, n5, n6));
    }
    
    public double ptSegDistSq(double n, double n2) {
        return ptSegDistSq(this.getX1(), this.getY1(), this.getX2(), this.getY2(), n, n2);
    }
    
    public double ptSegDistSq(Point2D point2D) {
        return ptSegDistSq(this.getX1(), this.getY1(), this.getX2(), this.getY2(), point2D.getX(), point2D.getY());
    }
    
    public double ptSegDist(double n, double n2) {
        return ptSegDist(this.getX1(), this.getY1(), this.getX2(), this.getY2(), n, n2);
    }
    
    public double ptSegDist(Point2D point2D) {
        return ptSegDist(this.getX1(), this.getY1(), this.getX2(), this.getY2(), point2D.getX(), point2D.getY());
    }
    
    public static double ptLineDistSq(double n, double n2, double n3, double n4, double n5, double n6) {
        n3 -= n;
        n4 -= n2;
        n5 -= n;
        n6 -= n2;
        double n7 = n5 * n3 + n6 * n4;
        double n8 = n5 * n5 + n6 * n6 - n7 * n7 / (n3 * n3 + n4 * n4);
        if (n8 < 0.0) {
            n8 = 0.0;
        }
        return n8;
    }
    
    public static double ptLineDist(double n, double n2, double n3, double n4, double n5, double n6) {
        return Math.sqrt(ptLineDistSq(n, n2, n3, n4, n5, n6));
    }
    
    public double ptLineDistSq(double n, double n2) {
        return ptLineDistSq(this.getX1(), this.getY1(), this.getX2(), this.getY2(), n, n2);
    }
    
    public double ptLineDistSq(Point2D point2D) {
        return ptLineDistSq(this.getX1(), this.getY1(), this.getX2(), this.getY2(), point2D.getX(), point2D.getY());
    }
    
    public double ptLineDist(double n, double n2) {
        return ptLineDist(this.getX1(), this.getY1(), this.getX2(), this.getY2(), n, n2);
    }
    
    public double ptLineDist(Point2D point2D) {
        return ptLineDist(this.getX1(), this.getY1(), this.getX2(), this.getY2(), point2D.getX(), point2D.getY());
    }
    
    @Override
    public boolean contains(double n, double n2) {
        return false;
    }
    
    @Override
    public boolean contains(Point2D point2D) {
        return false;
    }
    
    @Override
    public boolean intersects(double n, double n2, double n3, double n4) {
        return this.intersects(new Rectangle2D.Double(n, n2, n3, n4));
    }
    
    @Override
    public boolean intersects(Rectangle2D rectangle2D) {
        return rectangle2D.intersectsLine(this.getX1(), this.getY1(), this.getX2(), this.getY2());
    }
    
    @Override
    public boolean contains(double n, double n2, double n3, double n4) {
        return false;
    }
    
    @Override
    public boolean contains(Rectangle2D rectangle2D) {
        return false;
    }
    
    @Override
    public Rectangle getBounds() {
        return this.getBounds2D().getBounds();
    }
    
    @Override
    public PathIterator getPathIterator(AffineTransform affineTransform) {
        return new LineIterator(this, affineTransform);
    }
    
    @Override
    public PathIterator getPathIterator(AffineTransform affineTransform, double n) {
        return new LineIterator(this, affineTransform);
    }
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    public static class Float extends Line2D implements Serializable
    {
        public float x1;
        public float y1;
        public float x2;
        public float y2;
        private static final long serialVersionUID = 6161772511649436349L;
        
        public Float() {
        }
        
        public Float(float n, float n2, float n3, float n4) {
            this.setLine(n, n2, n3, n4);
        }
        
        public Float(Point2D point2D, Point2D point2D2) {
            super.setLine(point2D, point2D2);
        }
        
        @Override
        public double getX1() {
            return (double)this.x1;
        }
        
        @Override
        public double getY1() {
            return (double)this.y1;
        }
        
        @Override
        public Point2D getP1() {
            return new Point2D.Float(this.x1, this.y1);
        }
        
        @Override
        public double getX2() {
            return (double)this.x2;
        }
        
        @Override
        public double getY2() {
            return (double)this.y2;
        }
        
        @Override
        public Point2D getP2() {
            return new Point2D.Float(this.x2, this.y2);
        }
        
        @Override
        public void setLine(double n, double n2, double n3, double n4) {
            this.x1 = (float)n;
            this.y1 = (float)n2;
            this.x2 = (float)n3;
            this.y2 = (float)n4;
        }
        
        public void setLine(float x1, float y1, float x2, float y2) {
            this.x1 = x1;
            this.y1 = y1;
            this.x2 = x2;
            this.y2 = y2;
        }
        
        @Override
        public Rectangle2D getBounds2D() {
            float n;
            float n2;
            if (this.x1 < this.x2) {
                n = this.x1;
                n2 = this.x2 - this.x1;
            }
            else {
                n = this.x2;
                n2 = this.x1 - this.x2;
            }
            float n3;
            float n4;
            if (this.y1 < this.y2) {
                n3 = this.y1;
                n4 = this.y2 - this.y1;
            }
            else {
                n3 = this.y2;
                n4 = this.y1 - this.y2;
            }
            return new Rectangle2D.Float(n, n3, n2, n4);
        }
    }
    
    public static class Double extends Line2D implements Serializable
    {
        public double x1;
        public double y1;
        public double x2;
        public double y2;
        private static final long serialVersionUID = 7979627399746467499L;
        
        public Double() {
        }
        
        public Double(double n, double n2, double n3, double n4) {
            this.setLine(n, n2, n3, n4);
        }
        
        public Double(Point2D point2D, Point2D point2D2) {
            super.setLine(point2D, point2D2);
        }
        
        @Override
        public double getX1() {
            return this.x1;
        }
        
        @Override
        public double getY1() {
            return this.y1;
        }
        
        @Override
        public Point2D getP1() {
            return new Point2D.Double(this.x1, this.y1);
        }
        
        @Override
        public double getX2() {
            return this.x2;
        }
        
        @Override
        public double getY2() {
            return this.y2;
        }
        
        @Override
        public Point2D getP2() {
            return new Point2D.Double(this.x2, this.y2);
        }
        
        @Override
        public void setLine(double x1, double y1, double x2, double y2) {
            this.x1 = x1;
            this.y1 = y1;
            this.x2 = x2;
            this.y2 = y2;
        }
        
        @Override
        public Rectangle2D getBounds2D() {
            double n;
            double n2;
            if (this.x1 < this.x2) {
                n = this.x1;
                n2 = this.x2 - this.x1;
            }
            else {
                n = this.x2;
                n2 = this.x1 - this.x2;
            }
            double n3;
            double n4;
            if (this.y1 < this.y2) {
                n3 = this.y1;
                n4 = this.y2 - this.y1;
            }
            else {
                n3 = this.y2;
                n4 = this.y1 - this.y2;
            }
            return new Rectangle2D.Double(n, n3, n2, n4);
        }
    }
}
